<?php
namespace BsPaySdk\Increment;

use BsPaySdk\BsPayClient;
use Bankcard;
use ReqSeqID;
use SequenceID;

/**
 * 增值汇分期功能模块
 *
 * @package BsPaySdk\Increment
 */
class Installment extends BsPayClient
{
    /**
     * 银行卡分期支付请求
     *
     * @param string $huifuId 商户号
     * @param string $merOrdId 商户流水号
     * @param string $transAmt 交易金额
     * @param Bankcard $bankcard 银行卡对象
     * @param string $idType 证件类型 01-身份证
     * @param string $idNo 证件号
     * @param string $instalmentsNum 分期数 3-三期 6-六期...
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Installment 调用结果在返回对象的 result 中
     */
    public function bankcardPaymentApply($huifuId,
                                         $merOrdId,
                                         $transAmt,
                                         $bankcard,
                                         $idType,
                                         $idNo,
                                         $instalmentsNum,
                                         $notifyUrl,
                                         $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true), // 自动生成请求流水号
                "req_date" => date("Ymd"),  // 自动生成交易日期
                "huifu_id" => $huifuId,
                "mer_ord_id" => $merOrdId,
                "trans_amt" => $transAmt,
                "card_name" => $bankcard->getOwner(),
                "card_no" => $bankcard->getNo(),
                "card_mobile" => $bankcard->getMobile(),
                "id_type" => $idType,
                "id_no" => $idNo,
                "instalments_num" => $instalmentsNum,
                "notify_url" => $notifyUrl,
                "terminal_device_info" => array(    //在用户未传入情况下默认填充
                    "trx_device_type" => "4",
                    "trx_device_ip" => "127.0.0.1"
                )
            ), $extendInfos);

        # 调用接口
        self::postRequest("top.bccredit.paymentApply", $trans_params);

        return $this;
    }

    /**
     * 银行卡分期支付确认
     *
     * @param string $huifuId 商户号
     * @param string $merOrdId 本次交易商户流水号
     * @param string $orgSeqId 原分期支付请求流水号
     * @param string $orgSeqDate 原分期支付请求日期，8位日期字符串，YYYYMMDD
     * @param string $verifyCode 短信验证码
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Installment 调用结果在返回对象的 result 中
     */
    public function bankcardPaymentConfirm($huifuId,
                                           $merOrdId,
                                           $orgSeqId,
                                           $orgSeqDate,
                                           $verifyCode,
                                           $notifyUrl,
                                           $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true), // 自动生成请求流水号
                "req_date" => date("Ymd"),  // 自动生成交易日期
                "huifu_id" => $huifuId,
                "mer_ord_id" => $merOrdId,
                "org_seq_id" => $orgSeqId,
                "org_seq_date" => $orgSeqDate,
                "verify_code" => $verifyCode,
                "notify_url" => $notifyUrl,
                "terminal_device_info" => array(    //在用户未传入情况下默认填充
                    "trx_device_type" => "4",
                    "trx_device_ip" => "127.0.0.1"
                )
            ), $extendInfos);

        # 调用接口
        self::postRequest("top.bccredit.paymentConfirm", $trans_params);

        return $this;
    }

    /**
     * 银行卡分期交易查询
     *
     * @param string $huifuId 商户号
     * @param string $reqDate 分期支付交易日期（确认交易的日期），8位日期字符串，YYYYMMDD
     * @param SequenceID $seqId 分期支付交易标识号（确认交易的标识号）, 下列标识号二选一
     *                          <ul>
     *                              <li>ReqSeqID - 请求流水号
     *                              <li>HfSeqID - 汇付全局流水号
     *                          <ul> 注：亦可直接参考接口文档使用 CommonSeqID 对象设置
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Installment 调用结果在返回对象的 result 中
     */
    public function bankcardPaymentQuery($huifuId,
                                         $reqDate,
                                         SequenceID $seqId,
                                         $extendInfos=array()
    ){
        # 交易标识号转化为对应的数据合并入请求
        $seqIdParam = array(
            $seqId->key() => $seqId->value()
        );

        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_date" => $reqDate,
            ), $seqIdParam, $extendInfos);

        # 调用接口
        self::postRequest("topqur.bcCreditPayQuery", $trans_params);

        return $this;
    }

    /**
     * 银行卡分期支付退款
     *
     * @param string $huifuId 商户号
     * @param string $merOrdId 本次交易商户流水号
     * @param string $ordAmt 退款金额
     * @param SequenceID $orgSeqId 原交易标识号，下列标识号三选一
     *                          <ul>
     *                              <li>ReqSeqID - 请求流水号
     *                              <li>HfSeqID - 汇付全局流水号
     *                              <li>PartyOrderSeqID - 支付凭证号
     *                          <ul> 注：亦可直接参考接口文档使用 CommonSeqID 对象设置
     * @param string $orgReqDate 原交易日期，8位日期字符串，YYYYMMDD
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Installment 调用结果在返回对象的 result 中
     */
    public function bankcardPaymentRefund($huifuId,
                                          $merOrdId,
                                          $ordAmt,
                                          SequenceID $orgSeqId,
                                          $orgReqDate,
                                          $notifyUrl,
                                          $extendInfos=array()
    ){
        # 交易标识号转化为对应的数据合并入请求
        if ($orgSeqId instanceof ReqSeqID) {
            $seqIdParam = array(
                # 这个接口的订单号没有按照预设的key值-req_seq_id 加入请求，所以在此做个转换
                "org_req_seq_id" => $orgSeqId->value(),
            );
        }
        else {
            $seqIdParam = array(
                $orgSeqId->key() => $orgSeqId->value()
            );
        }

        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true), // 自动生成请求流水号
                "req_date" => date("Ymd"),  // 自动生成交易日期
                "huifu_id" => $huifuId,
                "mer_ord_id" => $merOrdId,
                "ord_amt" => $ordAmt,
                "org_req_date" => $orgReqDate,
                "notify_url" => $notifyUrl,
            ), $seqIdParam, $extendInfos);

        # 调用接口
        self::postRequest("toprrc.bccredit.refund", $trans_params);

        return $this;
    }

    /**
     * 银行卡分期签约
     *
     * @param string $huifuId 商户号
     * @param string $transAmt 交易金额 (单位元，需保留小数点后两位，例如：1.00,最低传入0.01)
     * @param Bankcard $bankcard 银行卡对象
     * @param string $idType 证件类型 01-身份证
     * @param string $idNo 证件号
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param string $businessUrl 页面跳转地址(支付成功后，前端跳转地址)
     * @param string $terminalType 发起请求的终端类型 (0:pc 1:安卓 2：ios)
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Installment 调用结果在返回对象的 result 中
     */
    public function bankcardSignup($huifuId,
                                   $transAmt,
                                   $bankcard,
                                   $idType,
                                   $idNo,
                                   $notifyUrl,
                                   $businessUrl,
                                   $terminalType,
                                   $extendInfos = array()
    ){
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true), // 自动生成请求流水号
                "req_date" => date("Ymd"),  // 自动生成交易日期
                "huifu_id" => $huifuId,
                "trans_amt" => $transAmt,
                "card_name" => $bankcard->getOwner(),
                "card_no" => $bankcard->getNo(),
                "card_mobile" => $bankcard->getMobile(),
                "id_type" => $idType,
                "id_no" => $idNo,
                "notify_url" => $notifyUrl,
                "business_url" => $businessUrl,
                "request_type" => $terminalType,
                "terminal_device_info" => array(    //在用户未传入情况下默认填充
                    "trx_device_type" => $terminalType,
                    "trx_device_ip" => "127.0.0.1"
                )
            ), $extendInfos);

        # 调用接口
        self::postRequest("top.bccredit.sign", $trans_params);

        return $this;
    }
}