<?php

namespace BsPaySdk\Online;

use Bankcard;
use BsPaySdk\BsPay;
use BsPaySdk\BsPayClient;
use BsPaySdk\utils\BsPayTools;
use Creditcard;
use IDInstance;
use PRCIDCard;

/**
 * 线上银行卡支付模块
 *
 * @package BsPaySdk\Online
 */
class CardPayment extends BsPayClient
{
    /**
     * 快捷代扣绑卡申请
     *
     * @param string $huifuId 商户号
     * @param string $merchName 商户名称（128）
     * @param string $verifyCardOrderId 一阶段验卡订单号
     * @param string $verifyCardOrderDate 一阶段验卡订单日期 YYYYMMDD
     * @param string $customId 客户用户号（64）
     * @param IDInstance $idInstance 身份证件（对象中的数据请明文传入，sdk将统一做加密后调用接口进行传输）
     *                            <ul>
     *                             <li>PRCIDCard - 中华人民共和国居民身份证（本方法传入的身份证对象需要包含完整的四要素）
     *                             <li>CommonID - 通用证件对象，详见接口文档说明
     *                            </ul>
     * @param Bankcard $bankcard 银行卡，Bankcard-银行卡对象/Creditcard-信用卡对象（对象中的数据请明文传入，sdk将统一做加密后调用接口进行传输）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function bind($huifuId,
                         $merchName,
                         $verifyCardOrderId,
                         $verifyCardOrderDate,
                         $customId,
                         IDInstance $idInstance,
                         Bankcard $bankcard,
                         $extendInfos = array()
    )
    {
        # 个人证件数据处理
        $idData = array(
            "cert_type" => ($idInstance instanceof PRCIDCard ? "00" : $idInstance->getType()),  // 这个接口的身份证标识是 00 和其他接口不同，后续更新迭代需要注意
            "cert_id" => BsPayTools::encrypt_with_rsa_pubkey(     // 对证件号码使用商户RSA加密
                BsPay::getHuifuPublicKey(),
                $idInstance->getIdNumber()
            ),
            "cert_validity_type" => $idInstance->getValidityType(),
            "cert_begin_date" => $idInstance->getValidityBeginDate(),
            "cert_end_date" => $idInstance->getValidityEndDate(),
        );

        # 银行卡数据处理
        $bankcardData = array(
            "card_id" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                BsPay::getHuifuPublicKey(),
                $bankcard->getNo()
            ),
            "card_name" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                BsPay::getHuifuPublicKey(),
                $bankcard->getOwner()
            ),
            "card_mp" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                BsPay::getHuifuPublicKey(),
                $bankcard->getMobile()
            )
        );
        # 添加信用卡数据字段
        if ($bankcard instanceof Creditcard) {
            $bankcardData = array_merge(array(
                "vip_code" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                    BsPay::getHuifuPublicKey(),
                    $bankcard->getCvv()
                ),
                "expiration" => BsPayTools::encrypt_with_rsa_pubkey(     // 对证件号码使用汇付公钥RSA加密
                    BsPay::getHuifuPublicKey(),
                    $bankcard->getExpiration()
                ),
            ), $bankcardData);
        }

        $req_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand()),     // 这个接口流水号长度最长只能送32位，这里减少随机参数
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "merch_name" => $merchName,
                "order_id" => $verifyCardOrderId,
                "order_date" => $verifyCardOrderDate,
                "out_cust_id" => $customId,
            ),
            $idData,
            $bankcardData,
            $extendInfos
        );

        # 调用接口
        self::postRequest("ssproxy.verifyCardApply", $req_params);

        return $this;
    }

    /**
     * 快捷代扣绑卡确认
     *
     * @param string $huifuId 商户号
     * @param string $merchName 商户名称（128）
     * @param string $verifyCardOrderId 一阶段验卡订单号
     * @param string $verifyCardOrderDate 一阶段验卡订单日期 YYYYMMDD
     * @param string $customId 客户用户号（64）
     * @param string $orgReqSeqId 原验卡请求流水号，
     * @param IDInstance $idInstance 身份证件（对象中的数据请明文传入，sdk将统一做加密后调用接口进行传输）
     *                            <ul>
     *                             <li>PRCIDCard - 中华人民共和国居民身份证（本方法传入的身份证对象需要包含完整的四要素）
     *                             <li>CommonID - 通用证件对象，详见接口文档说明
     *                            </ul>
     * @param Bankcard $bankcard 银行卡，Bankcard-银行卡对象/Creditcard-信用卡对象（对象中的数据请明文传入，sdk将统一做加密后调用接口进行传输）
     * @param string $verifyCode 短信验证码
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function bindConfirm($huifuId,
                                $merchName,
                                $verifyCardOrderId,
                                $verifyCardOrderDate,
                                $customId,
                                $orgReqSeqId,
                                IDInstance $idInstance,
                                Bankcard $bankcard,
                                $verifyCode,
                                $extendInfos = array()
    )
    {
        # 银行卡数据处理
        $bankcardData = array(
            "card_id" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                BsPay::getHuifuPublicKey(),
                $bankcard->getNo()
            ),
            "card_name" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                BsPay::getHuifuPublicKey(),
                $bankcard->getOwner()
            ),
            "card_mp" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                BsPay::getHuifuPublicKey(),
                $bankcard->getMobile()
            )
        );
        # 添加信用卡数据字段
        if ($bankcard instanceof Creditcard) {
            $bankcardData = array_merge(array(
                "vip_code" => BsPayTools::encrypt_with_rsa_pubkey(     // 对数据使用汇付公钥RSA加密
                    BsPay::getHuifuPublicKey(),
                    $bankcard->getCvv()
                ),
                "expiration" => BsPayTools::encrypt_with_rsa_pubkey(     // 对证件号码使用商户RSA加密
                    BsPay::getHuifuPublicKey(),
                    $bankcard->getExpiration()
                ),
            ), $bankcardData);
        }

        # 个人证件数据处理
        $idData = array(
            "cert_type" => ($idInstance instanceof PRCIDCard ? "00" : $idInstance->getType()),  // 这个接口的身份证标识是 00 和其他接口不同，后续更新迭代需要注意
            "cert_id" => BsPayTools::encrypt_with_rsa_pubkey(     // 对证件号码使用商户RSA加密
                BsPay::getHuifuPublicKey(),
                $idInstance->getIdNumber()
            ),
            "cert_validity_type" => $idInstance->getValidityType(),
            "cert_begin_date" => $idInstance->getValidityBeginDate(),
            "cert_end_date" => $idInstance->getValidityEndDate(),
        );

        $req_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand()),     // 这个接口流水号长度最长只能送32位，这里减少随机参数
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "merch_name" => $merchName,
                "order_id" => $verifyCardOrderId,
                "order_date" => $verifyCardOrderDate,
                "out_cust_id" => $customId,
                "trans_id" => $orgReqSeqId,     // 这个在接口文档标注的是非必填字段，但实际是必填的，---comment by Charles.huang 2021/08/16
                "verify_code" => $verifyCode
            ),
            $idData,
            $bankcardData,
            $extendInfos
        );

        # 调用接口
        self::postRequest("ssproxy.verifyCardConfirm", $req_params);

        return $this;
    }

    /**
     * 绑卡短信重发
     *
     * @param string $huifuId 商户号
     * @param string $verifyCardOrderId 一阶段验卡订单号
     * @param string $verifyCardOrderDate 一阶段验卡订单日期 YYYYMMDD
     * @param string $customId 客户用户号（64）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function resendBindSms($huifuId,
                                  $verifyCardOrderId,
                                  $verifyCardOrderDate,
                                  $customId,
                                  $extendInfos = array()
    )
    {
        $req_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand()),     // 这个接口流水号长度最长只能送32位，这里减少随机参数
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "order_id" => $verifyCardOrderId,
                "order_date" => $verifyCardOrderDate,
                "out_cust_id" => $customId,
            ),
            $extendInfos
        );

        # 调用接口
        self::postRequest("ssproxy.quickSendSms", $req_params);

        return $this;
    }

    /**
     * 快捷代扣解绑
     *
     * @param string $huifuId 商户号
     * @param string $merchName 商户名称（128）
     * @param string $customId 客户用户号（64）
     * @param string $tokenNo 卡令牌（20）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function unbind($huifuId,
                           $merchName,
                           $customId,
                           $tokenNo,
                           $extendInfos = array()
    )
    {
        $req_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand()),     // 这个接口流水号长度最长只能送32位，这里减少随机参数
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "merch_name" => $merchName,
                "out_cust_id" => $customId,
                "token_no" => $tokenNo,
            ),
            $extendInfos
        );

        # 调用接口
        self::postRequest("ssproxy.unBind", $req_params);

        return $this;
    }

    /**
     * 快捷支付申请
     *
     * @param string $huifuId 商户号
     * @param string $userHuifuId 快捷用户编号
     * @param string $transAmt 交易金额 (单位元，需保留小数点后两位，例如：1.00,最低传入0.01)
     * @param string $cardBindId 绑卡ID
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function quickPay($huifuId,
                             $userHuifuId,
                             $transAmt,
                             $cardBindId,
                             $notifyUrl,
                             $extendInfos = array()
    )
    {
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true), // 自动生成请求流水号
                "req_date" => date("Ymd"),  // 自动生成交易日期
                "huifu_id" => $huifuId,
                "user_huifu_id" => $userHuifuId,
                "trans_amt" => $transAmt,
                "card_bind_id" => $cardBindId,
                "mer_notice_url" => $notifyUrl,
                "device_info" => array(    //在用户未传入情况下默认填充
                    "trx_device_type" => "01",
                    "trx_device_ip" => "127.0.0.1"
                )
            ), $extendInfos);

        # 调用接口
        self::postRequest("top-online-ser.quickpay.apply", $trans_params);

        return $this;
    }

    /**
     * 快捷支付确认
     *
     * @param string $huifuId 商户号
     * @param string $orgSeqId 快捷支付申请流水号
     * @param string $orgSeqDate 快捷支付申请日期，原分期支付请求日期，8位日期字符串，YYYYMMDD
     * @param string $verifyCode 短信验证码
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function quickPayConfirm($huifuId,
                                    $orgSeqId,
                                    $orgSeqDate,
                                    $verifyCode,
                                    $notifyUrl,
                                    $extendInfos = array()
    )
    {
        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_seq_id" => $orgSeqId,
                "req_date" => $orgSeqDate,
                "sms_code" => $verifyCode,
                "mer_notice_url" => $notifyUrl,
            ), $extendInfos);

        # 调用接口
        self::postRequest("top-online-ser.quickpay.confirm", $trans_params);

        return $this;
    }

    /**
     * 快捷支付短信重发
     * @param array $params
     */
    public function resendQuickPaySms($params = array())
    {
        // TODO
    }

    /**
     * 代扣支付
     *
     * @param string $huifuId 商户号
     * @param string $userHuifuId 快捷用户编号
     * @param string $transAmt 交易金额 (单位元，需保留小数点后两位，例如：1.00,最低传入0.01)
     * @param string $cardBindId 绑卡ID
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function deductionPay($huifuId,
                                 $userHuifuId,
                                 $transAmt,
                                 $cardBindId,
                                 $notifyUrl,
                                 $extendInfos = array()
    )
    {
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true), // 自动生成请求流水号
                "req_date" => date("Ymd"),  // 自动生成交易日期
                "huifu_id" => $huifuId,
                "user_huifu_id" => $userHuifuId,
                "trans_amt" => $transAmt,
                "card_bind_id" => $cardBindId,
                "mer_notice_url" => $notifyUrl,
                "device_info" => array(    //在用户未传入情况下默认填充
                    "trx_device_type" => "01",
                    "trx_device_ip" => "127.0.0.1"
                )
            ), $extendInfos);

        # 调用接口
        self::postRequest("top-online-ser.withholdpay", $trans_params);

        return $this;
    }

    /**
     * 银行卡交易支付查询
     *
     * @param string $huifuId 商户号
     * @param string $orgReqDate 原交易发生日期
     * @param string $orgReqSeqId 原交易流水号
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function query($huifuId,
                          $orgReqDate,
                          $orgReqSeqId,
                          $extendInfos = array()
    )
    {
        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_date" => $orgReqDate,
                "req_seq_id" => $orgReqSeqId,
            ), $extendInfos);

        # 调用接口
        self::postRequest("top-online-ser.transQuery", $trans_params);

        return $this;
    }

    /**
     * 银行卡交易支付退款
     *
     * @param string $huifuId 商户号
     * @param string $ordAmt 退款金额
     * @param string $orgReqDate 原交易发生日期
     * @param string $orgReqSeqId 原交易流水号
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return CardPayment 调用结果在返回对象的 result 中
     */
    public function refund($huifuId,
                           $ordAmt,
                           $orgReqDate,
                           $orgReqSeqId,
                           $notifyUrl,
                           $extendInfos = array()
    )
    {
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "ord_amt" => $ordAmt,
                "org_req_date" => $orgReqDate,
                "org_req_seq_id" => $orgReqSeqId,
                "notify_url" => $notifyUrl,
            ), $extendInfos);

        # 调用接口
        self::postRequest("top-online-ser.refund", $trans_params);

        return $this;
    }

    /**
     * 银行卡交易支付退款查询
     * @param array $params
     */
    public function refundQuery($params = array())
    {
        // TODO
    }

    /**
     * 批量查询
     * @param array $params
     */
    public function queryBatch($params = array())
    {
        // TODO
    }
}