<?php
namespace BsPaySdk\Online;

use ActiveScanPayChannel;
use BsPaySdk\BsPayClient;
use ReqSeqID;
use SequenceID;

/**
 * 线上扫码支付模块
 *
 * @package BsPaySdk\Online
 */
class ScanPayment extends BsPayClient
{
    /**
     * 聚合正扫支付
     *
     * @param string $huifuId 商户号
     * @param string $transAmt 交易金额
     * @param string $merOrdId 商户交易流水号
     * @param string $goodsDesc 商品信息
     * @param ActiveScanPayChannel $activeScanPayChannelInfo 支付通道信息
     *                                                  <ul>
     *                                                   <li>ChannelJSAlipay - 支付宝js支付
     *                                                   <li>ChannelJSUnionpay - 银联js支付
     *                                                   <li>ChannelJSWechat - 微信js支付
     *                                                   <li>ChannelMiniWechat - 微信小程序支付
     *                                                   <li>ChannelNativeAlipay - 支付宝app支付
     *                                                   <li>ChannelNativeUnionpay - 银联app支付
     *                                                   <li>ChannelNativeWechat - 微信app支付
     *                                                  </ul>
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function activeScanPay($huifuId,
                                  $transAmt,
                                  $merOrdId,
                                  $goodsDesc,
                                  ActiveScanPayChannel $activeScanPayChannelInfo,
                                  $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "trans_amt" => $transAmt,
                "mer_ord_id" => $merOrdId,
                "goods_desc" => $goodsDesc,
                "trade_type" => $activeScanPayChannelInfo->getChannel(),    // 设置支付渠道类型
                $activeScanPayChannelInfo->getReqKey() => $activeScanPayChannelInfo->getChannelData()   // 添加支付渠道数据
            ), $extendInfos);

        # 调用接口
        self::postRequest("top.trans.pullPayInfo", $trans_params);

        return $this;
    }

    /**
     * 聚合反扫支付
     *
     * @param string $huifuId 商户号
     * @param string $transAmt 交易金额
     * @param string $merOrdId 商户交易流水号
     * @param string $authCode 支付授权码（扫码结果字符串）
     * @param string $goodsDesc 商品信息
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function passiveScanPay($huifuId,
                                   $transAmt,
                                   $merOrdId,
                                   $authCode,
                                   $goodsDesc,
                                   $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "huifu_id" => $huifuId,
                "trans_amt" => $transAmt,
                "mer_ord_id" => $merOrdId,
                "auth_code" => $authCode,
                "goods_desc" => $goodsDesc,
            ), $extendInfos);

        # 调用接口
        self::postRequest("top.trans.authCodePay", $trans_params);

        return $this;
    }

    /**
     * 扫码支付交易查询
     *
     * @param string $huifuId 商户号
     * @param string $orgReqDate 原交易日期
     * @param SequenceID $orgSeqId 原交易标识号，下列标识号五选一
     *                          <ul>
     *                              <li>ReqSeqID - 请求流水号
     *                              <li>HfSeqID - 汇付全局流水号
     *                              <li>PartyOrderSeqID - 支付凭证号
     *                              <li>AtTransOrderSeqID - 微信支付宝的订单号
     *                              <li>OutOrderSeqID - 商户外部订单号
     *                          <ul> 注：亦可直接参考接口文档使用 CommonSeqID 对象设置
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function query($huifuId,
                          $orgReqDate,
                          SequenceID $orgSeqId,
                          $extendInfos=array()
    ){
        # 交易标识号转化为对应的数据合并入请求
        $orgSeqIdParam = array(
            $orgSeqId->key() => $orgSeqId->value()
        );

        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_date" => $orgReqDate,
            ), $orgSeqIdParam, $extendInfos);

        # 调用接口
        self::postRequest("topqur.transQuery", $trans_params);

        return $this;
    }

    /**
     * 扫码支付交易退款
     *
     * @param string $huifuId 商户号
     * @param string $merOrdId 商户订单号
     * @param string $orgReqDate 原交易请求日期
     * @param SequenceID $orgSeqId 原交易标识号，下列标识号三选一
     *                          <ul>
     *                              <li>ReqSeqID - 请求流水号
     *                              <li>HfSeqID - 汇付全局流水号
     *                              <li>PartyOrderSeqID - 支付凭证号（支付宝微信）
     *                          <ul> 注：亦可直接参考接口文档使用 CommonSeqID 对象设置
     * @param string $ordAmt
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function refund($huifuId,
                           $merOrdId,
                           $orgReqDate,
                           SequenceID $orgSeqId,
                           $ordAmt,
                           $extendInfos=array()
    ){
        # 交易标识号转化为对应的数据合并入请求
        if ($orgSeqId instanceof ReqSeqID) {
            $orgSeqIdParam = array(
                # 这个接口的订单号没有按照预设的key值-req_seq_id 加入请求，所以在此做个转换
                "org_req_seq_id" => $orgSeqId->value(),
            );
        }
        else {
            $orgSeqIdParam = array(
                $orgSeqId->key() => $orgSeqId->value()
            );
        }

        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "mer_ord_id" => $merOrdId,
                "org_req_date" => $orgReqDate,
                "ord_amt" => $ordAmt,
            ),
            $orgSeqIdParam,
            $extendInfos
        );

        # 调用接口
        self::postRequest("toprrc.refund", $trans_params);

        return $this;
    }

    /**
     * 扫码支付交易退款查询
     *
     * @param string $huifuId 商户号
     * @param string $orgReqDate 原交易日期
     * @param SequenceID $orgSeqId 原交易标识号，下列标识号四选一
     *                          <ul>
     *                              <li>MerOrderSeqID - 原退款商户订单号
     *                              <li>ReqSeqID - 原始退款请求流水号
     *                              <li>HfSeqID - 原退款汇付流水号
     *                              <li>OutOrderSeqID - 原退款商户外部订单号
     *                          <ul> 注：亦可直接参考接口文档使用 CommonSeqID 对象设置
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function refundQuery($huifuId,
                                $orgReqDate,
                                SequenceID $orgSeqId,
                                $extendInfos=array()
    ){
        # 交易标识号转化为对应的数据合并入请求
        $orgSeqIdParam = array(
            $orgSeqId->key() => $orgSeqId->value()
        );

        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_date" => $orgReqDate,
            ), $orgSeqIdParam, $extendInfos);

        # 调用接口
        self::postRequest("topqur.refundQuery", $trans_params);

        return $this;
    }

    /**
     * 扫码支付交易关单
     *
     * @param string $huifuId 商户号
     * @param string $orgReqDate 原交易请求流水号
     * @param string $orgSeqId 原交易请求日期，"YYYYMMDD"
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function close($huifuId,
                          $orgReqDate,
                          $orgSeqId,
                          $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "org_req_seq_id" => $orgSeqId,
                "org_req_date" => $orgReqDate,
            ), $extendInfos);

        # 调用接口
        self::postRequest("toprrc.close", $trans_params);

        return $this;
    }

    /**
     * 扫码支付交易关单查询
     * @param array $params
     */
    public function closeQuery($params=array()){
        // TODO
    }

    /**
     * 获取银联用户标识
     *
     * @param string $huifuId 商户号
     * @param string $userAuthCode 用户授权码
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return ScanPayment 调用结果在返回对象的 result 中
     */
    public function getUnionId($huifuId,
                               $userAuthCode,
                               $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "user_auth_code" => $userAuthCode,
            ), $extendInfos);

        # 调用接口
        self::postRequest("top.trans.getUserMark", $trans_params);

        return $this;
    }
}