<?php
namespace BsPaySdk\Settlement;

use BankAcctDetails;
use BsPaySdk\BsPayClient;
use ReqSeqID;
use SequenceID;

/**
 * 取现与结算模块
 *
 * @package BsPaySdk\Settlement
 */
class Cashout extends BsPayClient
{
    /**
     * 代理商或商户或用户账户取现。
     * 可指定账户取现，不在额外参数中传入到账日期类型(into_acct_date_type: "T0"/"T1")的话，默认 "T0" 取现
     *
     * @param string $huifuId 商户号
     * @param string $merOrdId 商户订单号
     * @param string $cashAmt 取现金额
     * @param string $tokenNo 取现卡绑卡id
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Cashout 调用结果在返回对象的 result 中
     */
    public function acctCashout($huifuId,
                                $merOrdId,
                                $cashAmt,
                                $tokenNo,
                                $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "mer_ord_id" => $merOrdId,
                "cash_amt" => $cashAmt,
                "token_no" => $tokenNo,
                "into_acct_date_type" => "T0"
            ),
            $extendInfos
        );

        # 调用接口
        self::postRequest("toppas.enchashment", $trans_params);

        return $this;
    }

    /**
     * 代发
     *
     * @param string $huifuId 商户号
     * @param string $cashAmt 代发金额
     * @param string $intoAcctDateType 到账日期类型："T0"-T0交易；"T1"-T1交易；"D1"-D1交易
     * @param string $notifyUrl 异步通知地址（http/https异步通知地址，必须以virgo://开头）
     * @param string $purpose 代发目的描述
     * @param BankAcctDetails $bankcardDetails 银行卡详细信息
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Cashout 调用结果在返回对象的 result 中
     */
    public function surrogate($huifuId,
                              $cashAmt,
                              $intoAcctDateType,
                              $notifyUrl,
                              $purpose,
                              BankAcctDetails $bankcardDetails,
                              $extendInfos=array()
    ){
        $trans_params = array_merge(
            array(  // 基本信息
                "huifu_id" => $huifuId,
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "cash_amt"=> $cashAmt,
                "into_acct_date_type" => $intoAcctDateType,
                "notify_url" => $notifyUrl,
                "purpose" => $purpose
            ),
            array(  // 银行卡详情
                "bank_account_mask" => $bankcardDetails->getBankAccountMask(),
                "bank_account_crypt" => $bankcardDetails->getBankAccountCrypt(),
                "bank_code" => $bankcardDetails->getBankCode(),
                "bank_account_name"=> $bankcardDetails->getBankAccountName(),
                "public_and_private_flag" => $bankcardDetails->getPublicAndPrivateFlag(),
                "province" => $bankcardDetails->getProvince(),
                "area" => $bankcardDetails->getArea(),
                "bank_acct_three_in_one" => $bankcardDetails->getThreeInOneCode()
            ),
            $extendInfos
        );

        # 调用接口
        self::postRequest("toppas.surrogate", $trans_params);

        return $this;
    }

    /**
     * 代理商或商户或用户账户取现和结算查询
     *
     * @param string $huifuId 商户号
     * @param string $transReqDate 原交易日期
     * @param SequenceID $transOrdId 原交易标识号，下列标识号二选一
     *                          <ul>
     *                              <li>ReqSeqID - 请求流水号
     *                              <li>HfSeqID - 汇付全局流水号
     *                          <ul> 注：亦可直接参考接口文档使用 CommonSeqID 对象设置
     * @param array $extendInfos 可选扩展参数，详见接口文档，提供更多的可选信息将能有效的提高接口调用速度和成功率
     *
     * @return Cashout 调用结果在返回对象的 result 中
     */
    public function query($huifuId,
                          $transReqDate,
                          SequenceID $transOrdId,
                          $extendInfos=array()
    ){
        # 交易标识号转化为对应的数据合并入请求
        if ($transOrdId instanceof ReqSeqID) {
            $transOrdIdParam = array(
                # 这个接口的订单号没有按照预设的key值-req_seq_id 加入请求，所以在此做个转换
                "trans_mer_ord_id" => $transOrdId->value(),
            );
        }
        else {
            $transOrdIdParam = array(
                $transOrdId->key() => $transOrdId->value()
            );
        }

        $trans_params = array_merge(
            array(
                "huifu_id" => $huifuId,
                "req_seq_id" => uniqid(rand(), true),
                "req_date" => date("Ymd"),
                "trans_req_date" => $transReqDate
            ),
            $transOrdIdParam,
            $extendInfos
        );

        # 调用接口
        self::postRequest("topqur.cashQuery", $trans_params);

        return $this;
    }

    /**
     * 代理商或商户或用户对账单下载（文件）
     * @param array $params
     */
    public function downloadStatements($params=array()){
        // TODO
    }
}